==> soa-5.0.0-src.zip: soa-5.0.0-src/soa-work/esb/checkout/product/rosetta/src/org/jboss/internal/soa/esb/rosetta/pooling/JmsSession.java <==
/*
* JBoss, Home of Professional Open Source
* Copyright 2006, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.internal.soa.esb.rosetta.pooling;

import java.io.Serializable;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashSet;

import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicSubscriber;

/**
 * Wrapper for JMS session class, responsible for tracking resources and the pooling.
 */
public class JmsSession implements Session
{
    /**
     * The connection pool.
     */
    private final JmsConnectionPool pool ;
    /**
     * The session delegate.
     */
    private final Session session ;
    /**
     * The pool instance id.
     */
    private final long id ;
    /**
     * The session acknowledge mode.
     */
    private final int acknowledgeMode ;
    /**
     * The requested acknowledge mode for this session.
     */
    private final int requestedAcknowledgeMode ;

    /**
     * Flag indicating whether this session is suspect or not.
     */
    private boolean suspect ;
    
    /**
     * The set of active queue browsers.
     */
    private HashSet<QueueBrowser> queueBrowserSet ;
    /**
     * The set of active message consumers.
     */
    private HashSet<MessageConsumer> messageConsumerSet ;
    /**
     * The set of active message producers.
     */
    private HashSet<MessageProducer> messageProducerSet ;
    
    /**
     * Create the session wrapper.
     * @param pool The pool associated with this session.
     * @param session The session delegate.
     * @param id The pool instance id.
     * @param requestedAcknowledgeMode The requested acknowledge mode for this session.
     * @throws JMSException
     */
    JmsSession(final JmsConnectionPool pool, final Session session, final long id, final int requestedAcknowledgeMode)
        throws JMSException
    {
        this.pool = pool ;
        this.id = id ;
        this.session = (Session)getExceptionHandler(pool, Session.class, session) ;
        this.requestedAcknowledgeMode = requestedAcknowledgeMode ;
        acknowledgeMode = session.getAcknowledgeMode() ;
        // Workaround for JBESB-1873
        if ("org.jboss.jms.client.JBossSession".equals(session.getClass().getName()))
        {
            SessionExecutorInit.initExecutor(session) ;
        }
    }

    public long getId()
    {
        return id ;
    }
    
    public void close() throws JMSException
    {
        session.close();
    }

    public void commit() throws JMSException
    {
        setSuspect(true) ;
        session.commit();
        setSuspect(false) ;
    }

    public QueueBrowser createBrowser(Queue arg0, String arg1)
            throws JMSException
    {
        return trackQueueBrowser(session.createBrowser(arg0, arg1));
    }

    public QueueBrowser createBrowser(Queue arg0) throws JMSException
    {
        return trackQueueBrowser(session.createBrowser(arg0));
    }

    public BytesMessage createBytesMessage() throws JMSException
    {
        associate() ;
        return session.createBytesMessage();
    }

    public MessageConsumer createConsumer(Destination arg0, String arg1,
            boolean arg2) throws JMSException
    {
        return trackMessageConsumer(session.createConsumer(arg0, arg1, arg2));
    }

    public MessageConsumer createConsumer(Destination arg0, String arg1)
            throws JMSException
    {
        return trackMessageConsumer(session.createConsumer(arg0, arg1));
    }

    public MessageConsumer createConsumer(Destination arg0) throws JMSException
    {
        return trackMessageConsumer(session.createConsumer(arg0));
    }

    public TopicSubscriber createDurableSubscriber(Topic arg0, String arg1,
            String arg2, boolean arg3) throws JMSException
    {
        return trackTopicSubscriber(session.createDurableSubscriber(arg0, arg1, arg2, arg3));
    }

    public TopicSubscriber createDurableSubscriber(Topic arg0, String arg1)
            throws JMSException
    {
        return trackTopicSubscriber(session.createDurableSubscriber(arg0, arg1));
    }

    public MapMessage createMapMessage() throws JMSException
    {
        associate() ;
        return session.createMapMessage();
    }

    public Message createMessage() throws JMSException
    {
        associate() ;
        return session.createMessage();
    }

    public ObjectMessage createObjectMessage() throws JMSException
    {
        associate() ;
        return session.createObjectMessage();
    }

    public ObjectMessage createObjectMessage(Serializable arg0)
            throws JMSException
    {
        associate() ;
        return session.createObjectMessage(arg0);
    }

    public MessageProducer createProducer(Destination arg0) throws JMSException
    {
        return trackMessageProducer(session.createProducer(arg0));
    }

    public Queue createQueue(String arg0) throws JMSException
    {
        associate() ;
        return session.createQueue(arg0);
    }

    public StreamMessage createStreamMessage() throws JMSException
    {
        associate() ;
        return session.createStreamMessage();
    }

    public TemporaryQueue createTemporaryQueue() throws JMSException
    {
        associate() ;
        return session.createTemporaryQueue();
    }

    public TemporaryTopic createTemporaryTopic() throws JMSException
    {
        associate() ;
        return session.createTemporaryTopic();
    }

    public TextMessage createTextMessage() throws JMSException
    {
        associate() ;
        return session.createTextMessage();
    }

    public TextMessage createTextMessage(String arg0) throws JMSException
    {
        associate() ;
        return session.createTextMessage(arg0);
    }

    public Topic createTopic(String arg0) throws JMSException
    {
        associate() ;
        return session.createTopic(arg0);
    }

    public int getAcknowledgeMode() throws JMSException
    {
        return acknowledgeMode;
    }

    public MessageListener getMessageListener() throws JMSException
    {
        associate() ;
        return session.getMessageListener();
    }

    public boolean getTransacted() throws JMSException
    {
        associate() ;
        return session.getTransacted();
    }

    public void recover() throws JMSException
    {
        associate() ;
        session.recover();
    }

    public void rollback() throws JMSException
    {
        setSuspect(true) ;
        session.rollback();
        setSuspect(false) ;
    }

    public void run()
    {
        session.run();
    }

    public void setMessageListener(MessageListener arg0) throws JMSException
    {
        associate() ;
        session.setMessageListener(arg0);
    }

    public void unsubscribe(String arg0) throws JMSException
    {
        associate() ;
        session.unsubscribe(arg0);
    }

    private synchronized QueueBrowser trackQueueBrowser(QueueBrowser queueBrowser)
        throws JMSException
    {
        associate() ;
        if (queueBrowserSet == null)
        {
            queueBrowserSet = new HashSet<QueueBrowser>() ;
        }
        final QueueBrowser result = getQueueBrowser(queueBrowser) ;
        queueBrowserSet.add(result) ;
        return (QueueBrowser)getExceptionHandler(pool, QueueBrowser.class, result) ;
    }

    private synchronized MessageConsumer trackMessageConsumer(MessageConsumer messageConsumer)
        throws JMSException
    {
        associate() ;
        if (messageConsumerSet == null)
        {
            messageConsumerSet = new HashSet<MessageConsumer>() ;
        }
        final MessageConsumer result = getMessageConsumer(messageConsumer) ;
        messageConsumerSet.add(result) ;
        return (MessageConsumer)getExceptionHandler(pool, MessageConsumer.class, result) ;
    }

    private synchronized TopicSubscriber trackTopicSubscriber(TopicSubscriber topicSubscriber)
        throws JMSException
    {
        associate() ;
        if (messageConsumerSet == null)
        {
            messageConsumerSet = new HashSet<MessageConsumer>() ;
        }
        final TopicSubscriber result = getTopicSubscriber(topicSubscriber) ;
        messageConsumerSet.add(result) ;
        return (TopicSubscriber)getExceptionHandler(pool, TopicSubscriber.class, result) ;
    }

    private synchronized MessageProducer trackMessageProducer(MessageProducer messageProducer)
        throws JMSException
    {
        associate() ;
        if (messageProducerSet == null)
        {
            messageProducerSet = new HashSet<MessageProducer>() ;
        }
        final MessageProducer result = getMessageProducer(messageProducer) ;
        messageProducerSet.add(result) ;
        return (MessageProducer)getExceptionHandler(pool, MessageProducer.class, result) ;
    }

    synchronized void releaseResources()
    {
        if (queueBrowserSet != null)
        {
            for(QueueBrowser queueBrowser: queueBrowserSet)
            {
                try {
                    queueBrowser.close() ;
                } catch (final Exception jmse) {} // ignore
            }
            queueBrowserSet = null ;
        }
        if (messageConsumerSet != null)
        {
            for(MessageConsumer messageConsumer: messageConsumerSet)
            {
                try {
                    messageConsumer.close() ;
                } catch (final Exception jmse) {} // ignore
            }
            messageConsumerSet = null ;
        }
        if (messageProducerSet != null)
        {
            for(MessageProducer messageProducer: messageProducerSet)
            {
                try {
                    messageProducer.close() ;
                } catch (final Exception jmse) {} // ignore
            }
            messageProducerSet = null ;
        }
    }

    protected QueueBrowser getQueueBrowser(QueueBrowser queueBrowser)
    {
        return queueBrowser ;
    }

    protected MessageConsumer getMessageConsumer(MessageConsumer messageConsumer)
    {
        return messageConsumer ;
    }

    protected TopicSubscriber getTopicSubscriber(TopicSubscriber topicSubscriber)
    {
        return topicSubscriber ;
    }

    protected MessageProducer getMessageProducer(MessageProducer messageProducer)
    {
        return messageProducer ;
    }

    protected void handleCloseSession(final JmsConnectionPool jmsConnectionPool)
    {
        jmsConnectionPool.handleCloseSession(this) ;
    }

    protected void handleReleaseSession(JmsConnectionPool jmsConnectionPool)
    {
        jmsConnectionPool.handleReleaseSession(this) ;
    }
    
    protected void associate()
        throws JMSException
    {
    }
    
    protected void setSuspect(final boolean suspect)
    {
        this.suspect = suspect ;
    }
    
    public boolean isSuspect()
    {
        return suspect ;
    }
    
    public int getRequestedAcknowledgeMode()
    {
        return requestedAcknowledgeMode ;
    }

    /**
     * Wrap the object in an exception handler.
     * @param pool The pool associated with this session.
     * @param instanceClass The interface type of the instance.
     * @param instance The instance
     * @return 
     */
    protected Object getExceptionHandler(final JmsConnectionPool pool,
            final Class<?> instanceClass, final Object instance)
    {
        final InvocationHandler handler = new ExceptionHandler(pool, instance) ;
        return Proxy.newProxyInstance(getClass().getClassLoader(), new Class[] {instanceClass}, handler);
    }
    
    /**
     * Handler responsible for intercepting JMS exceptions and checking for unreported closure.
     * @author <a href='mailto:kevin.conner@jboss.com'>Kevin Conner</a>
     */
    private final class ExceptionHandler implements InvocationHandler
    {
        /**
         * The connection pool.
         */
        private final JmsConnectionPool pool ;
        /**
         * The target instance.
         */
        private final Object target ;
        
        /**
         * Construct the handler using the specified target.
         * @param pool The associated connection pool.
         * @param target The target instance.
         */
        public ExceptionHandler(final JmsConnectionPool pool, final Object target)
        {
            this.pool = pool ;
            this.target = target ;
        }
        
        public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
        {
            try
            {
                return method.invoke(target, args);
            }
            catch (final InvocationTargetException ite)
            {
                final Throwable th = ite.getCause() ;
                if (th instanceof JMSException)
                {
                    pool.handleException(JmsSession.this, (JMSException)th) ;
                }
                throw th ;
            }
        }
    }
}

==> soa-5.0.2-src.zip: soa-5.0.2-src/soa-work/esb/checkout/product/rosetta/src/org/jboss/internal/soa/esb/rosetta/pooling/JmsSession.java <==
/*
* JBoss, Home of Professional Open Source
* Copyright 2006, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.internal.soa.esb.rosetta.pooling;

import java.io.Serializable;
import java.lang.reflect.InvocationHandler;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Proxy;
import java.util.HashSet;

import javax.jms.BytesMessage;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MapMessage;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.ObjectMessage;
import javax.jms.Queue;
import javax.jms.QueueBrowser;
import javax.jms.Session;
import javax.jms.StreamMessage;
import javax.jms.TemporaryQueue;
import javax.jms.TemporaryTopic;
import javax.jms.TextMessage;
import javax.jms.Topic;
import javax.jms.TopicSubscriber;

import org.jboss.internal.soa.esb.rosetta.pooling.JmsConnectionPool.JmsSessionPool;

/**
 * Wrapper for JMS session class, responsible for tracking resources and the pooling.
 */
public class JmsSession implements Session
{
    /**
     * The connection pool.
     */
    private final JmsConnectionPool connectionPool ;
    /**
     * The session pool.
     */
    private final JmsSessionPool sessionPool ;
    /**
     * The session delegate.
     */
    private final Session session ;
    /**
     * The pool instance id.
     */
    private final long id ;
    /**
     * The session acknowledge mode.
     */
    private final int acknowledgeMode ;
    /**
     * The requested acknowledge mode for this session.
     */
    private final int requestedAcknowledgeMode ;

    /**
     * Flag indicating whether this session is suspect or not.
     */
    private boolean suspect ;
    
    /**
     * The set of active queue browsers.
     */
    private HashSet<QueueBrowser> queueBrowserSet ;
    /**
     * The set of active message consumers.
     */
    private HashSet<MessageConsumer> messageConsumerSet ;
    /**
     * The set of active message producers.
     */
    private HashSet<MessageProducer> messageProducerSet ;
    
    /**
     * Create the session wrapper.
     * @param connectionPool The connection pool associated with this session.
     * @param sessionPool The session pool associated with this session.
     * @param session The session delegate.
     * @param id The pool instance id.
     * @param requestedAcknowledgeMode The requested acknowledge mode for this session.
     * @throws JMSException
     */
    JmsSession(final JmsConnectionPool connectionPool, final JmsSessionPool sessionPool, final Session session, final long id, final int requestedAcknowledgeMode)
        throws JMSException
    {
        this.connectionPool = connectionPool ;
        this.sessionPool = sessionPool ;
        this.id = id ;
        this.session = (Session)getExceptionHandler(connectionPool, Session.class, session) ;
        this.requestedAcknowledgeMode = requestedAcknowledgeMode ;
        acknowledgeMode = session.getAcknowledgeMode() ;
        // Workaround for JBESB-1873
        if ("org.jboss.jms.client.JBossSession".equals(session.getClass().getName()))
        {
            SessionExecutorInit.initExecutor(session) ;
        }
    }

    public long getId()
    {
        return id ;
    }
    
    public void close() throws JMSException
    {
        session.close();
    }

    public void commit() throws JMSException
    {
        setSuspect(true) ;
        session.commit();
        setSuspect(false) ;
    }

    public QueueBrowser createBrowser(Queue arg0, String arg1)
            throws JMSException
    {
        return trackQueueBrowser(session.createBrowser(arg0, arg1));
    }

    public QueueBrowser createBrowser(Queue arg0) throws JMSException
    {
        return trackQueueBrowser(session.createBrowser(arg0));
    }

    public BytesMessage createBytesMessage() throws JMSException
    {
        associate() ;
        return session.createBytesMessage();
    }

    public MessageConsumer createConsumer(Destination arg0, String arg1,
            boolean arg2) throws JMSException
    {
        return trackMessageConsumer(session.createConsumer(arg0, arg1, arg2));
    }

    public MessageConsumer createConsumer(Destination arg0, String arg1)
            throws JMSException
    {
        return trackMessageConsumer(session.createConsumer(arg0, arg1));
    }

    public MessageConsumer createConsumer(Destination arg0) throws JMSException
    {
        return trackMessageConsumer(session.createConsumer(arg0));
    }

    public TopicSubscriber createDurableSubscriber(Topic arg0, String arg1,
            String arg2, boolean arg3) throws JMSException
    {
        return trackTopicSubscriber(session.createDurableSubscriber(arg0, arg1, arg2, arg3));
    }

    public TopicSubscriber createDurableSubscriber(Topic arg0, String arg1)
            throws JMSException
    {
        return trackTopicSubscriber(session.createDurableSubscriber(arg0, arg1));
    }

    public MapMessage createMapMessage() throws JMSException
    {
        associate() ;
        return session.createMapMessage();
    }

    public Message createMessage() throws JMSException
    {
        associate() ;
        return session.createMessage();
    }

    public ObjectMessage createObjectMessage() throws JMSException
    {
        associate() ;
        return session.createObjectMessage();
    }

    public ObjectMessage createObjectMessage(Serializable arg0)
            throws JMSException
    {
        associate() ;
        return session.createObjectMessage(arg0);
    }

    public MessageProducer createProducer(Destination arg0) throws JMSException
    {
        return trackMessageProducer(session.createProducer(arg0));
    }

    public Queue createQueue(String arg0) throws JMSException
    {
        associate() ;
        return session.createQueue(arg0);
    }

    public StreamMessage createStreamMessage() throws JMSException
    {
        associate() ;
        return session.createStreamMessage();
    }

    public TemporaryQueue createTemporaryQueue() throws JMSException
    {
        associate() ;
        return session.createTemporaryQueue();
    }

    public TemporaryTopic createTemporaryTopic() throws JMSException
    {
        associate() ;
        return session.createTemporaryTopic();
    }

    public TextMessage createTextMessage() throws JMSException
    {
        associate() ;
        return session.createTextMessage();
    }

    public TextMessage createTextMessage(String arg0) throws JMSException
    {
        associate() ;
        return session.createTextMessage(arg0);
    }

    public Topic createTopic(String arg0) throws JMSException
    {
        associate() ;
        return session.createTopic(arg0);
    }

    public int getAcknowledgeMode() throws JMSException
    {
        return acknowledgeMode;
    }

    public MessageListener getMessageListener() throws JMSException
    {
        associate() ;
        return session.getMessageListener();
    }

    public boolean getTransacted() throws JMSException
    {
        associate() ;
        return session.getTransacted();
    }

    public void recover() throws JMSException
    {
        associate() ;
        session.recover();
    }

    public void rollback() throws JMSException
    {
        setSuspect(true) ;
        session.rollback();
        setSuspect(false) ;
    }

    public void run()
    {
        session.run();
    }

    public void setMessageListener(MessageListener arg0) throws JMSException
    {
        associate() ;
        session.setMessageListener(arg0);
    }

    public void unsubscribe(String arg0) throws JMSException
    {
        associate() ;
        session.unsubscribe(arg0);
    }

    private synchronized QueueBrowser trackQueueBrowser(QueueBrowser queueBrowser)
        throws JMSException
    {
        associate() ;
        if (queueBrowserSet == null)
        {
            queueBrowserSet = new HashSet<QueueBrowser>() ;
        }
        final QueueBrowser result = getQueueBrowser(queueBrowser) ;
        queueBrowserSet.add(result) ;
        return (QueueBrowser)getExceptionHandler(connectionPool, QueueBrowser.class, result) ;
    }

    private synchronized MessageConsumer trackMessageConsumer(MessageConsumer messageConsumer)
        throws JMSException
    {
        associate() ;
        if (messageConsumerSet == null)
        {
            messageConsumerSet = new HashSet<MessageConsumer>() ;
        }
        final MessageConsumer result = getMessageConsumer(messageConsumer) ;
        messageConsumerSet.add(result) ;
        return (MessageConsumer)getExceptionHandler(connectionPool, MessageConsumer.class, result) ;
    }

    private synchronized TopicSubscriber trackTopicSubscriber(TopicSubscriber topicSubscriber)
        throws JMSException
    {
        associate() ;
        if (messageConsumerSet == null)
        {
            messageConsumerSet = new HashSet<MessageConsumer>() ;
        }
        final TopicSubscriber result = getTopicSubscriber(topicSubscriber) ;
        messageConsumerSet.add(result) ;
        return (TopicSubscriber)getExceptionHandler(connectionPool, TopicSubscriber.class, result) ;
    }

    private synchronized MessageProducer trackMessageProducer(MessageProducer messageProducer)
        throws JMSException
    {
        associate() ;
        if (messageProducerSet == null)
        {
            messageProducerSet = new HashSet<MessageProducer>() ;
        }
        final MessageProducer result = getMessageProducer(messageProducer) ;
        messageProducerSet.add(result) ;
        return (MessageProducer)getExceptionHandler(connectionPool, MessageProducer.class, result) ;
    }

    synchronized void releaseResources()
    {
        if (queueBrowserSet != null)
        {
            for(QueueBrowser queueBrowser: queueBrowserSet)
            {
                try {
                    queueBrowser.close() ;
                } catch (final Exception jmse) {} // ignore
            }
            queueBrowserSet = null ;
        }
        if (messageConsumerSet != null)
        {
            for(MessageConsumer messageConsumer: messageConsumerSet)
            {
                try {
                    messageConsumer.close() ;
                } catch (final Exception jmse) {} // ignore
            }
            messageConsumerSet = null ;
        }
        if (messageProducerSet != null)
        {
            for(MessageProducer messageProducer: messageProducerSet)
            {
                try {
                    messageProducer.close() ;
                } catch (final Exception jmse) {} // ignore
            }
            messageProducerSet = null ;
        }
    }

    protected QueueBrowser getQueueBrowser(QueueBrowser queueBrowser)
    {
        return queueBrowser ;
    }

    protected MessageConsumer getMessageConsumer(MessageConsumer messageConsumer)
    {
        return messageConsumer ;
    }

    protected TopicSubscriber getTopicSubscriber(TopicSubscriber topicSubscriber)
    {
        return topicSubscriber ;
    }

    protected MessageProducer getMessageProducer(MessageProducer messageProducer)
    {
        return messageProducer ;
    }

    protected void handleCloseSession(final JmsConnectionPool jmsConnectionPool)
    {
        jmsConnectionPool.handleCloseSession(this) ;
    }

    protected void handleReleaseSession(JmsConnectionPool jmsConnectionPool)
    {
        jmsConnectionPool.handleReleaseSession(this) ;
    }
    
    protected void associate()
        throws JMSException
    {
    }
    
    protected JmsSessionPool getSessionPool()
    {
        return sessionPool ;
    }
    
    protected void setSuspect(final boolean suspect)
    {
        this.suspect = suspect ;
    }
    
    public boolean isSuspect()
    {
        return suspect ;
    }
    
    public int getRequestedAcknowledgeMode()
    {
        return requestedAcknowledgeMode ;
    }

    /**
     * Wrap the object in an exception handler.
     * @param connectionPool The connection pool associated with this session.
     * @param instanceClass The interface type of the instance.
     * @param instance The instance
     * @return 
     */
    protected Object getExceptionHandler(final JmsConnectionPool connectionPool,
            final Class<?> instanceClass, final Object instance)
    {
        final InvocationHandler handler = new ExceptionHandler(connectionPool, instance) ;
        return Proxy.newProxyInstance(getClass().getClassLoader(), new Class[] {instanceClass}, handler);
    }
    
    /**
     * Handler responsible for intercepting JMS exceptions and checking for unreported closure.
     * @author <a href='mailto:kevin.conner@jboss.com'>Kevin Conner</a>
     */
    private final class ExceptionHandler implements InvocationHandler
    {
        /**
         * The connection pool.
         */
        private final JmsConnectionPool pool ;
        /**
         * The target instance.
         */
        private final Object target ;
        
        /**
         * Construct the handler using the specified target.
         * @param pool The associated connection pool.
         * @param target The target instance.
         */
        public ExceptionHandler(final JmsConnectionPool pool, final Object target)
        {
            this.pool = pool ;
            this.target = target ;
        }
        
        public Object invoke(Object proxy, Method method, Object[] args) throws Throwable
        {
            try
            {
                return method.invoke(target, args);
            }
            catch (final InvocationTargetException ite)
            {
                final Throwable th = ite.getCause() ;
                if (th instanceof JMSException)
                {
                    pool.handleException(JmsSession.this, (JMSException)th) ;
                }
                throw th ;
            }
        }
    }
}
